@@ -1,5 +1,4 @@
 {-# LANGUAGE CPP #-}
-{-# OPTIONS_GHC -fno-warn-unused-do-bind #-}
 --------------------------------------------------------------------------------
 -- |
 --  Module      :  Network.URI
@@ -80,6 +79,10 @@ module Network.URI
     , isIPv6address
     , isIPv4address
       
+    -- * Predicates
+    , uriIsAbsolute
+    , uriIsRelative
+      
     -- * Relative URIs
     , relativeTo
     , nonStrictRelativeTo
@@ -96,6 +99,7 @@ module Network.URI
     , uriToString
     , isReserved, isUnreserved
     , isAllowedInURI, isUnescapedInURI
+    , isUnescapedInURIComponent
     , escapeURIChar
     , escapeURIString
     , unEscapeString
@@ -122,6 +126,7 @@ import Text.ParserCombinators.Parsec
 
 import Control.Monad (MonadPlus(..))
 import Data.Char (ord, chr, isHexDigit, toLower, toUpper, digitToInt)
+import Data.Bits ((.|.),(.&.),shiftL,shiftR)
 import Debug.Trace (trace)
 import Numeric (showIntAtBase)
 
@@ -155,7 +160,7 @@ data URI = URI
     , uriPath       :: String           -- ^ @\/ghc@
     , uriQuery      :: String           -- ^ @?query@
     , uriFragment   :: String           -- ^ @#frag@
-    } deriving (Eq
+    } deriving (Eq, Ord
 #ifdef __GLASGOW_HASKELL__
     , Typeable, Data
 #endif
@@ -174,7 +179,7 @@ data URIAuth = URIAuth
     { uriUserInfo   :: String           -- ^ @anonymous\@@
     , uriRegName    :: String           -- ^ @www.haskell.org@
     , uriPort       :: String           -- ^ @:42@
-    } deriving (Eq
+    } deriving (Eq, Ord, Show
 #ifdef __GLASGOW_HASKELL__
     , Typeable, Data
 #endif
@@ -331,6 +336,16 @@ parseAll parser filename uristr = parse newparser filename uristr
                 ; return res
                 }
 
+------------------------------------------------------------
+--  Predicates
+------------------------------------------------------------
+
+uriIsAbsolute :: URI -> Bool
+uriIsAbsolute (URI {uriScheme = scheme}) = scheme /= ""
+
+uriIsRelative :: URI -> Bool
+uriIsRelative = not . uriIsAbsolute
+
 ------------------------------------------------------------
 --  URI parser body based on Parsec elements and combinators
 ------------------------------------------------------------
@@ -577,8 +592,10 @@ ipv4address =
         ; a2 <- decOctet ; char '.'
         ; a3 <- decOctet ; char '.'
         ; a4 <- decOctet
+        ; notFollowedBy regName
         ; return $ a1++"."++a2++"."++a3++"."++a4
         }
+    <?> "IPv4 Address"
 
 decOctet :: URIParser String
 decOctet =
@@ -890,6 +907,11 @@ isAllowedInURI c = isReserved c || isUnreserved c || c == '%' -- escape char
 isUnescapedInURI :: Char -> Bool
 isUnescapedInURI c = isReserved c || isUnreserved c
 
+-- | Returns 'True' if the character is allowed unescaped in a URI component.
+--
+isUnescapedInURIComponent :: Char -> Bool
+isUnescapedInURIComponent c = not (isReserved c || not (isUnescapedInURI c))
+
 ------------------------------------------------------------
 --  Escape sequence handling
 ------------------------------------------------------------
@@ -900,7 +922,7 @@ isUnescapedInURI c = isReserved c || isUnreserved c
 escapeURIChar :: (Char->Bool) -> Char -> String
 escapeURIChar p c
     | p c       = [c]
-    | otherwise = '%' : myShowHex (ord c) ""
+    | otherwise = concatMap (\i -> '%' : myShowHex i "") (utf8EncodeChar c)
     where
         myShowHex :: Int -> ShowS
         myShowHex n r =  case showIntAtBase 16 (toChrHex) n r of
@@ -911,6 +933,29 @@ escapeURIChar p c
             | d < 10    = chr (ord '0' + fromIntegral d)
             | otherwise = chr (ord 'A' + fromIntegral (d - 10))
 
+-- From http://hackage.haskell.org/package/utf8-string
+-- by Eric Mertens, BSD3
+-- Returns [Int] for use with showIntAtBase
+utf8EncodeChar :: Char -> [Int]
+utf8EncodeChar = map fromIntegral . go . ord
+ where
+  go oc
+   | oc <= 0x7f       = [oc]
+
+   | oc <= 0x7ff      = [ 0xc0 + (oc `shiftR` 6)
+                        , 0x80 + oc .&. 0x3f
+                        ]
+
+   | oc <= 0xffff     = [ 0xe0 + (oc `shiftR` 12)
+                        , 0x80 + ((oc `shiftR` 6) .&. 0x3f)
+                        , 0x80 + oc .&. 0x3f
+                        ]
+   | otherwise        = [ 0xf0 + (oc `shiftR` 18)
+                        , 0x80 + ((oc `shiftR` 12) .&. 0x3f)
+                        , 0x80 + ((oc `shiftR` 6) .&. 0x3f)
+                        , 0x80 + oc .&. 0x3f
+                        ]
+
 -- |Can be used to make a string valid for use in a URI.
 --
 escapeURIString
@@ -943,7 +988,7 @@ unEscapeString (c:s) = c : unEscapeString s
 --  Algorithm from RFC3986 [3], section 5.2.2
 --
 
-nonStrictRelativeTo :: URI -> URI -> Maybe URI
+nonStrictRelativeTo :: URI -> URI -> URI
 nonStrictRelativeTo ref base = relativeTo ref' base
     where
         ref' = if uriScheme ref == uriScheme base
@@ -953,9 +998,11 @@ nonStrictRelativeTo ref base = relativeTo ref' base
 isDefined :: ( MonadPlus m, Eq (m a) ) => m a -> Bool
 isDefined a = a /= mzero
 
--- |Compute an absolute 'URI' for a supplied URI
---  relative to a given base.
-relativeTo :: URI -> URI -> Maybe URI
+-- | Returns a new 'URI' which represents the value of the first 'URI'
+-- interpreted as relative to the second 'URI'.
+--
+-- Algorithm from RFC3986 [3], section 5.2
+relativeTo :: URI -> URI -> URI
 relativeTo ref base
     | isDefined ( uriScheme ref ) =
         just_segments ref
@@ -988,7 +1035,7 @@ relativeTo ref base
             }
     where
         just_segments u =
-            Just $ u { uriPath = removeDotSegments (uriPath u) }
+            u { uriPath = removeDotSegments (uriPath u) }
         mergePaths b r
             | isDefined (uriAuthority b) && null pb = '/':pr
             | otherwise                             = dropLast pb ++ pr
