@@ -32,9 +32,11 @@ import Network.URI
     , parseURI, parseURIReference, parseRelativeReference, parseAbsoluteURI
     , parseAbsoluteURI
     , isURI, isURIReference, isRelativeReference, isAbsoluteURI
+    , uriIsAbsolute, uriIsRelative
     , relativeTo, nonStrictRelativeTo
     , relativeFrom
     , uriToString
+    , isUnescapedInURIComponent
     , isUnescapedInURI, escapeURIString, unEscapeString
     , normalizeCase, normalizeEscape, normalizePathSegments
     )
@@ -213,6 +215,11 @@ testURIRef114 = testURIRef InvRf "http://example.org/xxx/qwerty#a#b"
 testURIRef115 = testURIRef InvRf "dcp.tcp.pft://192.168.0.1:1002:3002?fec=1&crc=0"
 testURIRef116 = testURIRef AbsId "dcp.tcp.pft://192.168.0.1:1002?fec=1&crc=0"
 testURIRef117 = testURIRef AbsId "foo://"
+-- URIs prefixed with IPv4 addresses
+testURIRef118 = testURIRef AbsId "http://192.168.0.1.example.com/"
+testURIRef119 = testURIRef AbsId "http://192.168.0.1.example.com./"
+-- URI prefixed with 3 octets of an IPv4 address and a subdomain part with a leading digit.
+testURIRef120 = testURIRef AbsId "http://192.168.0.1test.example.com/"
 
 testURIRefSuite = TF.testGroup "Test URIrefs" testURIRefList
 testURIRefList =
@@ -333,6 +340,10 @@ testURIRefList =
   , TF.testCase "testURIRef115" testURIRef115
   , TF.testCase "testURIRef116" testURIRef116
   , TF.testCase "testURIRef117" testURIRef117
+    --
+  , TF.testCase "testURIRef118" testURIRef118
+  , TF.testCase "testURIRef119" testURIRef119
+  , TF.testCase "testURIRef120" testURIRef120
   ]
 
 -- test decomposition of URI into components
@@ -429,11 +440,9 @@ testRelJoin  :: String -> String -> String -> String -> Assertion
 testRelJoin label base urel uabs =
     testEq label uabs (mkabs purel pubas)
     where
-        mkabs (Just u1) (Just u2) = shabs (u1 `relativeTo` u2)
+        mkabs (Just u1) (Just u2) = show (u1 `relativeTo` u2)
         mkabs Nothing   _         = "Invalid URI: "++urel
         mkabs _         Nothing   = "Invalid URI: "++uabs
-        shabs (Just u) = show u
-        shabs Nothing  = "No result"
         purel = parseURIReference urel
         pubas = parseURIReference base
 
@@ -1055,12 +1064,21 @@ testEscapeURIString03 = testEq "testEscapeURIString03"
 testEscapeURIString04 = testEq "testEscapeURIString04"
     te02str (unEscapeString te02esc)
 
+testEscapeURIString05 = testEq "testEscapeURIString05"
+    "http%3A%2F%2Fexample.org%2Faz%2F09-_%2F.~%3A%2F%3F%23%5B%5D%40%21%24%26%27%28%29%2A%2B%2C%3B%3D"
+    (escapeURIString isUnescapedInURIComponent te01str)
+
+testEscapeURIString06 = testEq "testEscapeURIString06"
+    "hello%C3%B8%C2%A9%E6%97%A5%E6%9C%AC"
+    (escapeURIString isUnescapedInURIComponent "helloø©日本")
 
 testEscapeURIString = TF.testGroup "testEscapeURIString"
   [ TF.testCase "testEscapeURIString01" testEscapeURIString01
   , TF.testCase "testEscapeURIString02" testEscapeURIString02
   , TF.testCase "testEscapeURIString03" testEscapeURIString03
   , TF.testCase "testEscapeURIString04" testEscapeURIString04
+  , TF.testCase "testEscapeURIString05" testEscapeURIString05
+  , TF.testCase "testEscapeURIString06" testEscapeURIString06
   ]
 
 -- URI string normalization tests
@@ -1114,17 +1132,17 @@ trbase = fromJust $ parseURIReference "http://bar.org/"
 
 testRelativeTo01 = testEq "testRelativeTo01"
     "http://bar.org/foo"
-    (show . fromJust $
+    (show $
       (fromJust $ parseURIReference "foo") `relativeTo` trbase)
 
 testRelativeTo02 = testEq "testRelativeTo02"
     "http:foo"
-    (show . fromJust $
+    (show $
       (fromJust $ parseURIReference "http:foo") `relativeTo` trbase)
 
 testRelativeTo03 = testEq "testRelativeTo03"
     "http://bar.org/foo"
-    (show . fromJust $
+    (show $
       (fromJust $ parseURIReference "http:foo") `nonStrictRelativeTo` trbase)
 
 testRelativeTo = TF.testGroup "testRelativeTo"
@@ -1177,6 +1195,31 @@ testAltFn = TF.testGroup "testAltFn"
   , TF.testCase "testAltFn17" testAltFn17
   ]
 
+testUriIsAbsolute :: String -> Assertion
+testUriIsAbsolute str =
+    assertBool str (uriIsAbsolute uri)
+    where
+    Just uri = parseURIReference str
+
+testUriIsRelative :: String -> Assertion
+testUriIsRelative str =
+    assertBool str (uriIsRelative uri)
+    where
+    Just uri = parseURIReference str
+
+testIsAbsolute = TF.testGroup "testIsAbsolute"
+  [ TF.testCase "testIsAbsolute01" $ testUriIsAbsolute "http://google.com"
+  , TF.testCase "testIsAbsolute02" $ testUriIsAbsolute "ftp://p.x.ca/woo?hai=a"
+  , TF.testCase "testIsAbsolute03" $ testUriIsAbsolute "mailto:bob@example.com"
+  ]
+
+testIsRelative = TF.testGroup "testIsRelative"
+  [ TF.testCase "testIsRelative01" $ testUriIsRelative "//google.com"
+  , TF.testCase "testIsRelative02" $ testUriIsRelative "/hello"
+  , TF.testCase "testIsRelative03" $ testUriIsRelative "this/is/a/path"
+  , TF.testCase "testIsRelative04" $ testUriIsRelative "?what=that"
+  ]
+
 -- Full test suite
 allTests =
   [ testURIRefSuite
@@ -1190,6 +1233,8 @@ allTests =
   , testNormalizeURIString
   , testRelativeTo
   , testAltFn
+  , testIsAbsolute
+  , testIsRelative
   ]
 
 main = TF.defaultMain allTests
@@ -1256,6 +1301,9 @@ cu02 = ou02 `relativeTo` bu02
 -- $Author: gklyne $
 -- $Revision: 1.8 $
 -- $Log: URITest.hs,v $
+-- Revision 1.81 2012/08/01           aaronfriel
+-- Added additional test case for the "xip.io" service style URLs and absolute URLs prefixed with ipv4 addresses.
+--
 -- Revision 1.8  2005/07/19 22:01:27  gklyne
 -- Added some additional test cases raised by discussion on URI@w3.org mailing list about 2005-07-19.  The test p[roposed by this discussion exposed a subtle bug in relativeFrom not being an exact inverse of relativeTo.
 --
